@@ -128,7 +128,7 @@ def validate_bool(b):
 
 
 def validate_bool_maybe_none(b):
-    'Convert b to a boolean or raise'
+    """Convert b to a boolean or raise."""
     if isinstance(b, str):
         b = b.lower()
     if b is None or b == 'none':
@@ -142,7 +142,7 @@ def validate_bool_maybe_none(b):
 
 
 def validate_float(s):
-    """convert s to float or raise"""
+    """Convert s to float or raise."""
     try:
         return float(s)
     except ValueError:
@@ -151,7 +151,7 @@ validate_floatlist = _listify_validator(validate_float)
 
 
 def validate_float_or_None(s):
-    """convert s to float, None or raise"""
+    """Convert s to float, None or raise."""
     # values directly from the rc file can only be strings,
     # so we need to recognize the string "None" and convert
     # it into the object. We will be case-sensitive here to
@@ -166,7 +166,7 @@ def validate_float_or_None(s):
 
 
 def validate_string_or_None(s):
-    """convert s to string or raise"""
+    """Convert s to string or raise."""
     if s is None:
         return None
     try:
@@ -202,7 +202,7 @@ def validate_axisbelow(s):
 
 
 def validate_dpi(s):
-    """confirm s is string 'figure' or convert s to float or raise"""
+    """Confirm s is string 'figure' or convert s to float or raise."""
     if s == 'figure':
         return s
     try:
@@ -213,7 +213,7 @@ def validate_dpi(s):
 
 
 def validate_int(s):
-    """convert s to int or raise"""
+    """Convert s to int or raise."""
     try:
         return int(s)
     except ValueError:
@@ -221,7 +221,7 @@ def validate_int(s):
 
 
 def validate_int_or_None(s):
-    """if not None, tries to validate as an int"""
+    """Return None if s is None or return ``int(s)``, otherwise raise."""
     if s == 'None':
         s = None
     if s is None:
@@ -234,8 +234,8 @@ def validate_int_or_None(s):
 
 def validate_fonttype(s):
     """
-    confirm that this is a Postscript of PDF font type that we know how to
-    convert to
+    Confirm that this is a Postscript or PDF font type that we know how to
+    convert to.
     """
     fonttypes = {'type3':    3,
                  'truetype': 42}
@@ -302,7 +302,7 @@ class validate_nseq_float(object):
         self.allow_none = allow_none
 
     def __call__(self, s):
-        """return a seq of n floats or raise"""
+        """Return a list of *n* floats or raise."""
         if isinstance(s, str):
             s = [x.strip() for x in s.split(',')]
             err_msg = _str_err_msg
@@ -326,7 +326,7 @@ class validate_nseq_int(object):
         self.n = n
 
     def __call__(self, s):
-        """return a seq of n ints or raise"""
+        """Return a list of *n* ints or raise."""
         if isinstance(s, str):
             s = [x.strip() for x in s.split(',')]
             err_msg = _str_err_msg
@@ -343,7 +343,7 @@ class validate_nseq_int(object):
 
 
 def validate_color_or_inherit(s):
-    'return a valid color arg'
+    """Return a valid color arg."""
     if s == 'inherit':
         return s
     return validate_color(s)
@@ -372,7 +372,7 @@ def validate_color_for_prop_cycle(s):
 
 
 def validate_color(s):
-    'return a valid color arg'
+    """Return a valid color arg."""
     try:
         if s.lower() == 'none':
             return 'none'
@@ -864,7 +864,7 @@ def cycler(*args, **kwargs):
 
 
 def validate_cycler(s):
-    'return a Cycler object from a string repr or the object itself'
+    """Return a Cycler object from a string repr or the object itself."""
     if isinstance(s, str):
         try:
             # TODO: We might want to rethink this...
